#! /usr/bin/env python3

# This script is meant to be used with the output from:
#   ./lingeling -v --no-elim --no-ternres --no-card --log=2 <CNF file>


import sys
from collections import deque
import re

extracted = []
inv_extracted = {}
leaf = False
prev = [-1, -1]
dag = {}
finished = False
time = -1
timeline = re.compile("c (\d+\.\d+) seconds")

with open(sys.argv[1], 'r') if len(sys.argv) > 1 else sys.stdin as f:
    for i, line_in in enumerate(f):
        line = line_in.strip()
        add = False
        match = timeline.match(line)
        if match:
            time = timeline.match(line).group(1)
        elif finished:
            continue
        elif line[:18] == 'c LOG2 0 extracted':
            # leaf in the dag
            lits_in, pos = line.split('constraint')[1].strip().split('=')
            lits = list(map(int, lits_in.split()))
            lits.sort()
            if int(pos.strip()) == 1:
                tup = tuple(lits)
                if tup not in extracted:
                    extracted.append(tup)
                    inv_extracted[str(tup)] = len(extracted) - 1
            else:
                tup = tuple([-lits[0]] + lits[1:])
                if tup not in extracted:
                    extracted.append(tup)
                    inv_extracted[str(tup)] = len(extracted) - 1
        elif line == 'c LOG1 0 inconsistent result row 0 = 1 from gaussian elimination':
            dag[-1] = (prev[0], prev[1])
            finished = True
        elif line[:21] == 'c LOG2 0   result row':
            # non-leaf node in the dag
            lits_in, pos = line.split('row')[1].strip().split('=')
            lits = list(map(int, lits_in.split()))
            lits.sort()
            if int(pos.strip()) == 1:
                tup = tuple(lits)
                if tup not in extracted:
                    extracted.append(tup)
                    inv_extracted[str(tup)] = len(extracted) - 1
                    dag[len(extracted) - 1] = (prev[0], prev[1])
            else:
                tup = tuple([-lits[0]] + lits[1:])
                if tup not in extracted:
                    extracted.append(tup)
                    inv_extracted[str(tup)] = len(extracted) - 1
                    dag[len(extracted) - 1] = (prev[0], prev[1])
        elif line[:25] == 'c LOG2 0   1st row (kept)':
            # information about child 1 of upcoming node
            lits_in, pos = line.split('(kept)')[1].strip().split('=')
            lits = list(map(int, lits_in.split()))
            lits.sort()
            if int(pos.strip()) == 1:
                tup = tuple(lits)
            else:
                tup = tuple([-lits[0]] + lits[1:])
            prev[0] = inv_extracted[str(tup)]
        elif line[:29] == 'c LOG2 0   2nd row (replaced)':
            # information about child 2 of upcoming node
            lits_in, pos = line.split('(replaced)')[1].strip().split('=')
            lits = list(map(int, lits_in.split()))
            lits.sort()
            if int(pos.strip()) == 1:
                tup = tuple(lits)
            else:
                tup = tuple([-lits[0]] + lits[1:])
            prev[1] = inv_extracted[str(tup)]
        else:
            continue
        

if -1 not in dag:
    print("c No inconsistent row found")
    exit(1)

print("c Gaussian elimination proof generated from Lingeling output")
print(f"c Lingeling solving time (sec): {time}")
print("c")

traversal = deque()
deq = deque([-1])
visited = set()

while deq:
    node = deq.popleft()
    if node >= 0:
        traversal.append(node)
    if node in dag:
        # the kept xor
        deq.append(dag[node][0])
        # the replaced xor
        deq.append(dag[node][1])
    
while traversal:
    delete = False
    xor_node = traversal.pop()
    xor = extracted[xor_node]
    if xor_node in dag:
        delete = True
        del_node = dag[xor_node][1]
        del_xor = extracted[del_node]
    print(f"x{' '.join(map(str, xor))} 0")
    if delete:
        print(f"d x{' '.join(map(str, del_xor))} 0")

print(0)
